package betterwithmods.event;

import betterwithmods.common.items.ItemBreedingHarness;
import betterwithmods.util.InvUtils;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLiving;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.SharedMonsterAttributes;
import net.minecraft.entity.passive.EntityCow;
import net.minecraft.entity.passive.EntityPig;
import net.minecraft.entity.passive.EntitySheep;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.network.datasync.DataParameter;
import net.minecraft.network.datasync.EntityDataManager;
import net.minecraft.util.EnumHand;
import net.minecraft.util.SoundCategory;
import net.minecraft.world.World;
import net.minecraftforge.event.entity.EntityEvent;
import net.minecraftforge.event.entity.EntityJoinWorldEvent;
import net.minecraftforge.event.entity.living.LivingEvent;
import net.minecraftforge.event.entity.player.PlayerInteractEvent;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.items.CapabilityItemHandler;
import net.minecraftforge.items.IItemHandlerModifiable;

import static net.minecraft.network.datasync.DataSerializers.ITEM_STACK;

/**
 * Purpose:
 *
 * @author primetoxinz
 * @version 11/27/16
 */
public class BreedingHardnessEvent {

    private static final String TAG_HARNESS = "betterwithmods:harness";

    private static final DataParameter<ItemStack> COW_DATA = EntityDataManager.func_187226_a(EntityCow.class, field_187196_f),
            PIG_DATA = EntityDataManager.func_187226_a(EntityPig.class, field_187196_f),
            SHEEP_DATA = EntityDataManager.func_187226_a(EntitySheep.class, field_187196_f);

    private static DataParameter<ItemStack> getHarnessData(Entity e) {
        if (e instanceof EntityCow)
            return COW_DATA;
        else if (e instanceof EntityPig)
            return PIG_DATA;
        else if (e.getClass() == EntitySheep.class)
            return SHEEP_DATA;
        else
            return null;
    }

    public static boolean isValidAnimal(Entity animal) {
        return animal instanceof EntityCow || animal instanceof EntityPig || animal.getClass() == EntitySheep.class;
    }

    public static ItemStack getHarness(EntityLivingBase living) {
        return living.func_184212_Q().func_187225_a(getHarnessData(living));
    }


    @SubscribeEvent
    public void onEntityInit(EntityEvent.EntityConstructing event) {
        if (isValidAnimal(event.getEntity())) {
            EntityDataManager manager = event.getEntity().func_184212_Q();
            manager.func_187214_a(getHarnessData(event.getEntity()), ItemStack.field_190927_a);
        }
    }

    @SubscribeEvent
    public void onEntity(EntityJoinWorldEvent e) {
        if (isValidAnimal(e.getEntity())) {
            EntityLiving animal = (EntityLiving) e.getEntity();
            ItemStack dataStack = animal.func_184212_Q().func_187225_a(getHarnessData(animal));

            ItemStack nbtStack = ItemStack.field_190927_a;
            if(animal.getEntityData().func_74764_b(TAG_HARNESS)) {
                NBTTagCompound cmp = animal.getEntityData().func_74775_l(TAG_HARNESS);
                if (!cmp.func_82582_d())
                    nbtStack = new ItemStack(cmp);
            }
            if (dataStack != nbtStack)
                e.getEntity().func_184212_Q().func_187227_b(getHarnessData(e.getEntity()), nbtStack);
        }
    }

    @SubscribeEvent
    public void onEntityInteract(PlayerInteractEvent.EntityInteract event) {
        if (event.getHand() != EnumHand.MAIN_HAND)
            return;
        Entity target = event.getTarget();
        EntityPlayer player = event.getEntityPlayer();
        World world = event.getWorld();
        if (isValidAnimal(target) && !target.func_184188_bt().contains(player)) {
            EntityLiving animal = (EntityLiving) target;
            IItemHandlerModifiable playerInv = (IItemHandlerModifiable) player.getCapability(CapabilityItemHandler.ITEM_HANDLER_CAPABILITY, null);

            ItemStack harness = getHarness(animal);
            ItemStack held = player.func_184614_ca();
            if (!harness.func_190926_b()) {
                if (held.func_190926_b()) {
                    InvUtils.insert(playerInv, harness, false);
                    animal.func_184212_Q().func_187227_b(getHarnessData(animal), ItemStack.field_190927_a);
                    animal.getEntityData().func_74782_a(TAG_HARNESS, new NBTTagCompound());
                    world.func_184133_a(null, animal.func_180425_c(), SoundEvents.field_187728_s, SoundCategory.NEUTRAL, 0.5f, 1.3f);
                    world.func_184133_a(null, animal.func_180425_c(), SoundEvents.field_187713_n, SoundCategory.NEUTRAL, 0.5f, 1.3f);
                }
            } else if (held.func_77973_b() instanceof ItemBreedingHarness) {
                if (!getHarness(animal).func_190926_b())
                    return;
                ItemStack copyStack = held.func_77946_l();
                InvUtils.consumeItemsInInventory(playerInv, held, 1, false);
                copyStack.func_190920_e(1);
                NBTTagCompound cmp = new NBTTagCompound();
                copyStack.func_77955_b(cmp);
                animal.func_184212_Q().func_187227_b(getHarnessData(animal), copyStack);
                animal.getEntityData().func_74782_a(TAG_HARNESS, cmp);
                if (animal.getClass() == EntitySheep.class)
                    ((EntitySheep) animal).func_70893_e(true);
                world.func_184133_a(null, animal.func_180425_c(), SoundEvents.field_187728_s, SoundCategory.NEUTRAL, 1, 1);
                world.func_184133_a(null, animal.func_180425_c(), SoundEvents.field_187713_n, SoundCategory.NEUTRAL, 1, 1f);
                player.func_184609_a(EnumHand.MAIN_HAND);
            }
        }
    }

    @SubscribeEvent
    public void onLivingTick(LivingEvent.LivingUpdateEvent e) {
        EntityLivingBase entity = e.getEntityLiving();
        if (isValidAnimal(entity)) {
            if (!getHarness(entity).func_190926_b())
                entity.func_110148_a(SharedMonsterAttributes.field_111263_d).func_111128_a(-1);
            else
                entity.func_110148_a(SharedMonsterAttributes.field_111263_d).func_111128_a(0.25);
        }
    }

}
